#include <regex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "landerctl.h"

static const char cfg_line_regex_expr[] = "^([^ ]+) *= *([^ ]+)$";

landerctl_cfg_err landerctl_cfg_parse(landerctl_cfg *out, const char *path) {
  FILE *f = fopen(path, "r");

  if (f == NULL) {
    return landerctl_cfg_err_not_found;
  }

  struct {
    const char *key;
    const char **var;
  } key_to_vars[] = {
      {"api_key", &out->api_key},
      {"server_url", &out->server_url},
      {"ca_certs_bundle", &out->ca_certs_bundle},
  };
  size_t key_to_vars_len = sizeof(key_to_vars) / sizeof(key_to_vars[0]);

  // We NULL everything beforehand so we can check if we have all needed
  // variables
  for (size_t i = 0; i < key_to_vars_len; i++) {
    *key_to_vars[i].var = NULL;
  }

  regex_t cfg_line_regex;
  regcomp(&cfg_line_regex, cfg_line_regex_expr, REG_EXTENDED);

  // Accept lines of at most 256 lines
  char line[256];
  landerctl_cfg_err res = landerctl_cfg_err_ok;

  while (fgets(line, sizeof(line), f) != NULL) {
    // Last character might be a newline
    size_t len = strlen(line);

    if (line[len - 1] == '\n') {
      line[len - 1] = '\0';
    }

    regmatch_t reg_groups[3];

    if (regexec(&cfg_line_regex, line, 3, reg_groups, 0) != 0) {
      res = landerctl_cfg_err_not_found;

      break;
    }

    // api_key is currently the only value we parse
    size_t key_len = reg_groups[1].rm_eo - reg_groups[1].rm_so;

    for (size_t i = 0; i < key_to_vars_len; i++) {
      if ((key_len == strlen(key_to_vars[i].key)) &&
          (strncmp(&line[reg_groups[1].rm_so], key_to_vars[i].key, key_len) ==
           0)) {
        int val_len = reg_groups[2].rm_eo - reg_groups[2].rm_so;
        char *buf = malloc(val_len + 1);
        strncpy(buf, &line[reg_groups[2].rm_so], val_len);
        buf[val_len] = '\0';

        *key_to_vars[i].var = buf;
        break;
      }
    }
  }

  if (res == landerctl_cfg_err_ok) {
    for (size_t i = 0; i < key_to_vars_len; i++) {
      if (*key_to_vars[i].var == NULL) {
        res = landerctl_cfg_err_incomplete;
        break;
      }
    }
  }

  fclose(f);

  return res;
}
