#include <assert.h>
#include <stdio.h>

#include "lsm.h"
#include "lsm/store.h"
#include "lsm/str.h"

int main() {
  lsm_str *data_dir;
  lsm_str_init_copy(&data_dir, "data");

  lsm_store *store;
  assert(lsm_store_load(&store, data_dir) == lsm_error_ok);

  lsm_str *key;
  lsm_str_init_copy(&key, "key");

  lsm_entry_handle *handle;
  assert(lsm_store_insert(&handle, store, key) == lsm_error_ok);

  lsm_str *attr;
  lsm_str_init_copy(&attr, "some attribute value");
  lsm_entry_attr_insert(handle, 1, attr);

  lsm_str *data;
  lsm_str_init_copy(&data, "hello");

  for (int i = 0; i < 50; i++) {
    lsm_entry_data_append(handle, data);
  }
  lsm_entry_close(handle);

  assert(lsm_store_open_read(&handle, store, key) == lsm_error_ok);

  char buf[24];
  uint64_t read;
  uint64_t total = 0;

  lsm_entry_data_read(&read, buf, handle, 24);
  total += read;

  while (read > 0) {
    printf("%.*s", read, buf);
    lsm_entry_data_read(&read, buf, handle, 24);
    total += read;
  }
  printf("\n%lu", total);

  lsm_entry_close(handle);

  assert(lsm_store_open_write(&handle, store, key) == lsm_error_ok);
  lsm_entry_remove(handle);
  lsm_entry_close(handle);

  assert(lsm_store_open_read(&handle, store, key) == lsm_error_not_found);
}
