#include "lsm/store.h"
#include "lsm/store_internal.h"

lsm_error lsm_store_iter(lsm_store_iterator **out, lsm_store *store,
                         const lsm_str *prefix) {
  lsm_trie_iterator *trie_iter;
  LSM_RES(lsm_trie_iter(&trie_iter, store->trie, prefix));

  lsm_store_iterator *iter = calloc(1, sizeof(lsm_store_iterator));

  if (iter == NULL) {
    lsm_trie_iter_free(trie_iter);

    return lsm_error_failed_alloc;
  }

  iter->iter = trie_iter;
  iter->store = store;

  *out = iter;

  return lsm_error_ok;
}

lsm_error lsm_store_iter_next_read(lsm_entry_handle **out,
                                   lsm_store_iterator *iter) {
  lsm_entry_wrapper *wrapper = NULL;

  // Traverse through the trie until a node is found with a filled data field
  do {
    // Exits function if iterator is done
    LSM_RES(lsm_trie_iter_next((void **)&wrapper, iter->iter));

    // TODO error handling?
    pthread_rwlock_rdlock(&wrapper->lock);

    if (wrapper->entry == NULL) {
      pthread_rwlock_unlock(&wrapper->lock);

      wrapper = NULL;
    }
  } while (wrapper == NULL);

  lsm_entry_handle *handle;
  LSM_RES2(lsm_entry_handle_init(&handle),
           pthread_rwlock_unlock(&wrapper->lock));

  handle->wrapper = wrapper;
  handle->store = iter->store;
  *out = handle;

  return lsm_error_ok;
}

void lsm_store_iter_free(lsm_store_iterator *iter) {
  lsm_trie_iter_free(iter->iter);
  free(iter);
}
