#ifndef LTM_TEMPLATE_INTERNAL
#define LTM_TEMPLATE_INTERNAL

#include "ltm/template.h"

typedef enum ltm_placeholder_type {
  ltm_placeholder_type_invalid = 0,
  ltm_placeholder_type_var,
  ltm_placeholder_type_nested_start,
  ltm_placeholder_type_nested_end,
} ltm_placeholder_type;

/**
 * Internal struct used when compiling templates
 */
typedef struct ltm_placeholder {
  const char *start;
  const char *end;
  struct {
    const char *s;
    size_t len;
  } name;
  ltm_placeholder_type type;
} ltm_placeholder;

/**
 * The type of a template block
 */
typedef enum ltm_template_block_type {
  ltm_template_block_type_literal = 0,
  ltm_template_block_type_var,
  ltm_template_block_type_nested,
} ltm_template_block_type;

/**
 * A block in a template.
 */
typedef struct ltm_template_block {
  ltm_template_block_type type;
  struct {
    void *ptr;
    size_t len;
  } data;
} ltm_template_block;

typedef struct ltm_template_block_name {
  struct {
    const char *s;
    size_t len;
  } name;
  size_t index;
} ltm_template_block_name;

/**
 * Represents a compiled template. A template consists of a list of blocks, of
 * which some have names that can be indexed (e.g. variables, nested templates).
 */
struct ltm_template {
  struct {
    ltm_template_block *arr;
    size_t len;
  } blocks;
  struct {
    ltm_template_block_name *arr;
    size_t len;
  } names;
};

/**
 * Allocate a new `ltm_template`.
 */
ltm_err ltm_template_init(ltm_template **out);

/**
 * Append a name to the template's list.
 */
ltm_err ltm_template_name_append(ltm_template *template, const char *s,
                                 size_t len, size_t index);

/**
 * Append a block to the template's list.
 */
ltm_err ltm_template_block_append(ltm_template *template,
                                  ltm_template_block_type type, void *data,
                                  size_t len);

typedef struct ltm_instance_block {
  ltm_instance_block_type type;
  ltm_data_fn fn;
  struct {
    void *ptr;
    size_t len;
  } data;
  struct ltm_instance_block *next;
} ltm_instance_block;

ltm_err ltm_instance_block_init(ltm_instance_block **out);

/**
 * Add a new block to the instance with the given type.
 */
ltm_err ltm_instance_block_add(ltm_instance_block **out, ltm_instance *instance,
                               const char *name, ltm_instance_block_type type);

struct ltm_instance {
  const ltm_template *template;
  struct {
    ltm_instance_block *head;
    ltm_instance_block *tail;
    ltm_instance_block *current;
  } blocks;
  ltm_instance_block ***vars;
  // How many bytes of the current block have been written
  size_t written;
};

ltm_err ltm_instance_init(ltm_instance **out);

#endif
