#include "ltm/template.h"
#include "ltm/template_internal.h"

ltm_err ltm_template_init(ltm_template **out) {
  ltm_template *template = calloc(1, sizeof(ltm_template));

  if (template == NULL) {
    return ltm_err_failed_alloc;
  }

  *out = template;

  return ltm_err_ok;
}

void ltm_template_free(ltm_template *template) {
  if (template->blocks.len > 0) {
    for (size_t i = 0; i < template->blocks.len; i++) {
      ltm_template_block *block = &template->blocks.arr[i];

      switch (block->type) {
      case ltm_template_block_type_nested:
        ltm_template_free(block->data.ptr);
        break;
      case ltm_template_block_type_var:
      case ltm_template_block_type_literal:;
      }
    }

    free(template->blocks.arr);
  }

  if (template->names.len > 0) {
    free(template->names.arr);
  }

  free(template);
}

ltm_err ltm_template_name_append(ltm_template *template, const char *s,
                                 size_t len, size_t index) {
  ltm_template_block_name *names =
      realloc(template->names.arr,
              (template->names.len + 1) * sizeof(ltm_template_block_name));

  if (names == NULL) {
    return ltm_err_failed_alloc;
  }

  ltm_template_block_name *name = &names[template->names.len];

  name->name.s = s;
  name->name.len = len;
  name->index = index;

  template->names.arr = names;
  template->names.len++;

  return ltm_err_ok;
}

ltm_err ltm_template_block_append(ltm_template *template,
                                  ltm_template_block_type type, void *data,
                                  size_t len) {
  ltm_template_block *blocks =
      realloc(template->blocks.arr,
              (template->blocks.len + 1) * sizeof(ltm_template_block));

  if (blocks == NULL) {
    return ltm_err_failed_alloc;
  }

  ltm_template_block *block = &blocks[template->blocks.len];

  block->type = type;

  switch (type) {
  case ltm_template_block_type_literal:
    block->data.ptr = data;
    block->data.len = len;
    break;
  case ltm_template_block_type_nested:
    block->data.ptr = data;
    break;
  // For the other cases, we explicitely ignore the data and len arguments
  default:;
  }

  template->blocks.arr = blocks;
  template->blocks.len++;

  return ltm_err_ok;
}
